@@ -74,11 +74,15 @@
   Added compatibility entries for sound, printer, video4linux, parallel port
   and frame buffer drivers.
 
-    Last updated by Richard Gooch   26-OCT-1999: Incremented protocol revision
+    Updated by      Richard Gooch   26-OCT-1999: Incremented protocol revision
   to 5. Extract major and minor number in <do_scan_and_service>. Added
   compatibility entries for SoundBlaster CD-ROM, netlink, SCSI generic and
   SCSI tape drivers.
 
+    Last updated by Richard Gooch   30-OCT-1999: Added entries for /dev/st,
+  /dev/sg and /dev/sr hierarchies. Added compatibility entries for loop, IDE
+  tape and SCSI CD-ROM drivers.
+
 
 */
 #include <stdio.h>
@@ -104,7 +108,7 @@
 #include <karma.h>
 
 
-#define VERSION "1.2.6"
+#define VERSION "1.2.7"
 #define CONFIG_FILE "/etc/devfsd.conf"
 #define MAX_ARGS (6 + 1)
 
@@ -764,8 +768,10 @@ static void action_compat (const struct devfsd_notify_struct *info,
 */
 {
     const char *compat_name = NULL;
+    const char *dest_name = info->devname;
     char *ptr;
     char compat_path[STRING_LENGTH], compat_buf[STRING_LENGTH];
+    char dest_buf[STRING_LENGTH];
     static char function_name[] = "action_compat";
 
     /*  First construct compatibility name  */
@@ -816,9 +822,84 @@ static void action_compat (const struct devfsd_notify_struct *info,
 	    compat_name = compat_buf;
 	    if (info->devname[info->namelen - 1] != 'n') ++compat_name;
 	}
+	else if (strncmp (info->devname, "loop/", 5) == 0)
+	{
+	    sprintf (compat_buf, "loop%s", info->devname + 5);
+	    compat_name = compat_buf;
+	}
+	else if (strncmp (info->devname, "ide/mt/", 7) == 0)
+	{
+	    char rewind = info->devname[info->namelen - 1];
+
+	    if (rewind != 'n') rewind = '\0';
+	    sprintf (compat_buf, "nht%u", info->minor & 0x7f);
+	    compat_name = compat_buf;
+	    if (rewind != 'n') ++compat_name;
+	}
+	else if ( (strncmp (info->devname, "scsi/", 4) == 0) &&
+		  (strcmp (info->devname + info->namelen - 2, "cd") == 0) )
+	{
+	    sprintf (compat_buf, "sr%u", info->minor);
+	    compat_name = compat_buf;
+	}
 	break;
       case AC_MKNEWCOMPAT:
       case AC_RMNEWCOMPAT:
+	if ( (strncmp (info->devname, "scsi/", 4) == 0) &&
+	     (strncmp (ptr = strrchr (info->devname, '/'), "mt", 2)
+	      == 0) )
+	{
+	    int mode, host, bus, target, lun;
+	    char rewind = info->devname[info->namelen - 1];
+
+	    if (rewind != 'n') rewind = '\0';
+	    switch (ptr[2])
+	    {
+	      default:
+		mode = 0;
+		break;
+	      case 'l':
+		mode = 1;
+		break;
+	      case 'm':
+		mode = 2;
+		break;
+	      case 'a':
+		mode = 3;
+		break;
+	    }
+	    sscanf (info->devname + 5, "host%d/bus%d/target%d/lun%d/",
+		    &host, &bus, &target, &lun);
+	    sprintf (compat_buf, "st/c%db%dt%du%dm%d%c",
+		     host, bus, target, lun, mode, rewind);
+	    compat_name = compat_buf;
+	    sprintf (dest_buf, "../%s", info->devname);
+	    dest_name = dest_buf;
+	}
+	else if ( (strncmp (info->devname, "scsi/", 4) == 0) &&
+		  (strcmp (info->devname + info->namelen - 7,"generic") == 0) )
+	{
+	    int host, bus, target, lun;
+
+	    sscanf (info->devname + 5, "host%d/bus%d/target%d/lun%d/",
+		    &host, &bus, &target, &lun);
+	    sprintf (compat_buf, "sg/c%db%dt%du%d", host, bus, target, lun);
+	    compat_name = compat_buf;
+	    sprintf (dest_buf, "../%s", info->devname);
+	    dest_name = dest_buf;
+	}
+	else if ( (strncmp (info->devname, "scsi/", 4) == 0) &&
+		  (strcmp (info->devname + info->namelen - 2, "cd") == 0) )
+	{
+	    int host, bus, target, lun;
+
+	    sscanf (info->devname + 5, "host%d/bus%d/target%d/lun%d/",
+		    &host, &bus, &target, &lun);
+	    sprintf (compat_buf, "sr/c%db%dt%du%d", host, bus, target, lun);
+	    compat_name = compat_buf;
+	    sprintf (dest_buf, "../%s", info->devname);
+	    dest_name = dest_buf;
+	}
 	break;
     }
     if (compat_name == NULL) return;
@@ -828,7 +909,7 @@ static void action_compat (const struct devfsd_notify_struct *info,
     {
       case AC_MKOLDCOMPAT:
       case AC_MKNEWCOMPAT:
-	if (symlink (info->devname, compat_path) != 0)
+	if (symlink (dest_name, compat_path) != 0)
 	{
 	    SYSLOG (LOG_ERR, "%s: error creating symlink: \"%s\"\t%s\n",
 		    function_name, compat_path, ERRSTRING);
